package org.rcisoft.business.bbanner.controller;


import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.annotations.Param;
import org.rcisoft.business.bbanner.dto.AddOrUpdateDTO;
import org.rcisoft.business.bbanner.dto.FindBannerPaginDTO;
import org.rcisoft.business.bbanner.dto.TopDownDTO;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bbanner.entity.BBanner;
import org.rcisoft.business.bbanner.service.BBannerService;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.rcisoft.core.result.Result;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @author
 * @date 2019/9/11
 */

@Api(tags = "01 Banner接口")
@RestController
@RequestMapping("/Banner")
@Slf4j
public class BBannerController extends PaginationController<BBanner> {

    @Autowired
    private BBannerService bBannerService;

    @ApiOperation(value = "101 添加/编辑Banner", notes = "businessId为空时是添加方法，不为空时是修改方法")
    @PostMapping(value = "/add")
    public Result add(CurUser curUser, @Valid AddOrUpdateDTO addOrUpdateDTO, BindingResult bindingResult) {
        PersistModel data = bBannerService.persist(addOrUpdateDTO);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                addOrUpdateDTO);
    }


    @ApiOperation(value = "102 删除Banner", notes = "根据ID删除一条记录")
    @ApiImplicitParam(name = "businessId", value = "主键id", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(CurUser curUser, String businessId, BindingResult bindingResult) {
        PersistModel data = bBannerService.removeBanner(businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }


    @ApiOperation(value = "103 唯一查找", notes = "根据ID查找一条记录", response = BBanner.class)
    @ApiImplicitParam(name = "businessId", value = "主键id", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(CurUser curUser, String businessId, BindingResult bindingResult) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bBannerService.selectOne(businessId));
    }


    @ApiOperation(value = "104 分页获取banner", notes = "根据条件分页查询", response = BBanner.class)
    @GetMapping(value = "/queryBBannerByPagination")
    public Result queryBBannerByPagination(CurUser curUser, @Valid FindBannerPaginDTO param, BindingResult bindingResult) {
        bBannerService.queryBannerByPagination(getPaginationUtility(), param);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value = "105 Banner重新排序", notes = "Banner重新排序")
    @PostMapping(value = "/updateBannerOrder")
    public Result updateBannerOrder(CurUser curUser, @Param("bannerOrderMap") String bannerOrderMap,BindingResult bindingResult){
        int data = bBannerService.updateBannerOrder(bannerOrderMap);
        return Result.builder(new PersistModel(data),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bannerOrderMap);
    }


}
