package org.rcisoft.business.bchapter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bchapter.dao.BPChapterRepository;
import org.rcisoft.business.bchapter.dto.*;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.enums.*;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bchapter.vo.StudentChapterScoreVO;
import org.rcisoft.business.bfile.dao.BPFileRepository;
import org.rcisoft.business.bfile.dto.uploadDTO;
import org.rcisoft.business.blesson.dao.*;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.bfile.dao.BFileRepository;
import org.rcisoft.business.bfile.entity.BFile;
import org.rcisoft.business.blesson.entity.BLessonPerson;
import org.rcisoft.business.blesson.entity.BPLesson;
import org.rcisoft.business.blesson.entity.BViewrange;
import org.rcisoft.business.blesson.enums.IsFinishEnum;
import org.rcisoft.business.blesson.enums.LessonSycnEnum;
import org.rcisoft.business.blesson.enums.LessonTypeEnum;
import org.rcisoft.business.blesson.enums.ReleaseStateEnum;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.business.blesson.service.impl.BLessonServiceImpl;
import org.rcisoft.business.blesson.util.QueryDepart;
import org.rcisoft.business.blessonlog.dao.BLessonLogRepository;
import org.rcisoft.business.blessonlog.entity.BLessonLog;
import org.rcisoft.business.bmaterial.dao.BMaterialRepository;
import org.rcisoft.business.bmaterial.entity.BMaterial;
import org.rcisoft.business.bmessage.dao.BMessageRepository;
import org.rcisoft.business.bpersonvalue.entity.BPersonValue;
import org.rcisoft.business.brstudentchapter.dao.BRStudentChapterRepository;
import org.rcisoft.business.brstudentchapter.entity.BRStudentChapter;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.util.feignDto.MTUserGetsReqDTO;
import org.rcisoft.common.util.feignDto.MTUserInfoRspDTO;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.DelStatus;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.*;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.URL;
import java.net.UnknownHostException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * Created by gwf on 2017-7-21 15:08:47.
 */
@Slf4j
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class BChapterServiceImpl implements BChapterService {
    @Autowired
    private BChapterRepository bChapterRepository;
    //    @Autowired
//    private BSlRepository bSlRepository;
//    @Autowired
//    private BStudentRepository bStudentRepository;
    @Autowired
    private BLessonRepository bLessonRepository;
    @Autowired
    private BFileRepository bFileRepository;
    @Autowired
    private BMaterialRepository bMaterialRepository;
    @Autowired
    private Global global;
    @Autowired
    BRStudentChapterRepository brStudentChapterRepository;
    @Autowired
    BLessonPersonRepository bLessonPersonRepository;
    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;
    @Autowired
    BCollectRepository bCollectRepository;
    @Autowired
    BLessonService bLessonService;
    @Autowired
    OfficeToPdf officeToPdf;
    @Autowired
    BLessonServiceImpl bLessonServiceImpl;
    @Autowired
    BMessageRepository bMessageRepository;
    @Autowired
    BLessonLogRepository bLessonLogRepository;

    @Autowired
    BViewrangeRepository bViewrangeRepository;

    @Autowired
    BPLessonRepository bpLessonRepository;

    @Autowired
    BPChapterRepository bpChapterRepository;

    @Autowired
    BPFileRepository bpFileRepository;

    @Value("${upload.fileMaxSize:100}")
    private Long fileMaxSize;

    @Value("${upload.fileMaxPages:100}")
    private Integer fileMaxPages;

    @Value("${upload.pdfCutPage:400}")
    private Integer pdfCutPage;

    @Value("${upload.pdfMaxSize:150}")
    private Long pdfMaxSize;

    @Value("${upload.txtMaxSize:10}")
    private Long txtMaxSize;

    @Value("${upload.replaceHttp:false}")
    private String replaceHttp;


    @Override
    public BChapter selectOne(String businessId) {
        BChapter result = bChapterRepository.selectByPrimaryKey(businessId);
        if (result == null)
            throw new ServiceException(ResultServiceEnums.CHAPTER_NOT_EXISTS);
        return result;
    }

    @Override
    public List<QueryChapterListResDTO> queryBChaptersByPagination(PageUtil pageUtil, QueryChapterListDTO model, String corpId) {
        return queryChapterListResDTO(model, corpId);
    }

    @Override
    public List<StudentChapterScoreVO> queryScoreListByChapterIdByPagination(PageUtil pageUtil, QueryScoreListDTO dto) {
        return bChapterRepository.queryScoreListByChapterId(dto);
    }

    @Override
    public List<QueryChapterListResDTO> queryBChapters(QueryChapterListDTO model, String corpId) {
        return queryChapterListResDTO(model, corpId);
    }

    @Override
    public Map queryBChaptersWithProgress(String lessonId, CurUser curUser, String isExternalShare, String chapterId) {

        List<BViewrange> bViewrangeList = bViewrangeRepository.selectByLessonId(lessonId);
        if (bViewrangeList != null && bViewrangeList.size() > 0 && "1".equals(isExternalShare)) {
            throw new ServiceException(ResultServiceEnums.NO_ACCESS);
        }

        QueryChapterListDTO model = new QueryChapterListDTO();
        model.setLessonId(lessonId);
        model.setStudentId(curUser.getUserId());
        List<QueryChapterListResDTO> queryChapterListResDTOS = queryChapterListResDTO(model, curUser.getCorpId());
        if (StringUtils.isNotEmpty(curUser.getUserId())) {
            for (QueryChapterListResDTO queryChapterListResDTO : queryChapterListResDTOS) {
                for (QueryChapterListResDTO childList : queryChapterListResDTO.getChildList()) {
                    // 遍历学生章节中间表 查询学生观看章节进度
                    List<BRStudentChapter> brStudentChapterList = brStudentChapterRepository.queryByStuIdAndChapter(childList.getBusinessId(), curUser.getUserId());
                    if (null != brStudentChapterList && brStudentChapterList.size() > 0) {
                        childList.setProgress(brStudentChapterList.get(0).getProgress() * 100 + "%");
                        childList.setCurrentLocation(brStudentChapterList.get(0).getCurrentLocation());
                    }
                }
            }
        }
        Map map = new HashMap();
        DecimalFormat df = new DecimalFormat("0.00");//设置保留位数
        map.put("chapterList", queryChapterListResDTOS);
        List<BLessonPerson> bLessonPersonList = bLessonPersonRepository.getAppointInTraining(lessonId, curUser.getUserId());
        if (null != bLessonPersonList && bLessonPersonList.size() > 0 && bLessonPersonList.get(0) != null) {
            map.put("totalProgress", bLessonPersonList.get(0).getLearnProgress());
        } else {
            map.put("totalProgress", "0%");
        }
        List<BLesson> bLessonList = bLessonRepository.selectInfoById(lessonId, curUser);
        //判断课程是否存在
        if (bLessonList == null || bLessonList.size() < 1 || bLessonList.get(0) == null) {
            throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
        }
        BLesson bLesson = bLessonList.get(0);
//        //判断课程是否被关闭
//        if (ReleaseStateEnum.CLOSED.getCode().equals(bLesson.getReleaseState())){
//            throw new ServiceException(ResultServiceEnums.LESSON_CLOSED);
//        }
        //判断该登录人是否可见该课程
        List<String> ids = Arrays.asList(new String[]{curUser.getUserId()});
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        List<String> departs = QueryDepart.queryDepart(mtUserInfoRspDTOList);
        int result = bLessonRepository.isInViewRange(curUser, lessonId, departs);
        if (result < 1 && !(null != bLessonPersonList && bLessonPersonList.size() > 0 && bLessonPersonList.get(0) != null && "1".equals(bLessonPersonList.get(0).getIsAppoint()))) {
            throw new ServiceException(ResultServiceEnums.NO_ACCESS);
        }

        //设置讲师部门
        List<String> lecturerIds = new ArrayList<>();
        lecturerIds.add(bLesson.getLecturerId());
        MTUserGetsReqDTO mtUserGetsReqDTOlecDept = new MTUserGetsReqDTO();
        mtUserGetsReqDTOlecDept.setCorpId(bLesson.getCorpId());
        mtUserGetsReqDTOlecDept.setIds(lecturerIds);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOLecDeptList = mtCotactApiRequestClient.userGets(mtUserGetsReqDTOlecDept);
        if (mtUserInfoRspDTOLecDeptList != null && mtUserInfoRspDTOLecDeptList.size() > 0) {
            mtUserInfoRspDTOLecDeptList.forEach(mtUserInfoRspDTO -> {
                if (mtUserInfoRspDTO.getDepts() != null && mtUserInfoRspDTO.getDepts().size() > 0 && mtUserInfoRspDTO.getId().equals(bLesson.getLecturerId())) {
                    //设置部门名
                    bLesson.setLecturerDeptName(mtUserInfoRspDTO.getDepts().get(0).getName());
                }
                if (mtUserInfoRspDTO.getId().equals(bLesson.getLecturerId()) && StringUtils.isNotEmpty(mtUserInfoRspDTO.getAvatar())) {
                    bLesson.setLecturerPic(mtUserInfoRspDTO.getAvatar());
                }
            });
        }

        //查询该课程是否收藏
        int collectCount = bCollectRepository.selectCountbyLessonAndUser(curUser, bLesson.getBusinessId());
        if (collectCount > 0) {
            bLesson.setIsCollect("1");
        } else {
            bLesson.setIsCollect("0");
        }
        //判断课程是否有可见范围（用于是否能外部分享）
        if (bViewrangeList != null && bViewrangeList.size() > 0) {
            bLesson.setHaveViewrange("1");
        }

        map.put("lessonInfo", bLesson);  // 课程信息

        QueryChapterListResDTO queryChapterListResDTO = bChapterRepository.getWithCurrentChapterBylessonId(lessonId, curUser.getUserId(), curUser.getCorpId());
        if (null != queryChapterListResDTO && StringUtils.isNotBlank(queryChapterListResDTO.getPid())) {
            //已加入我学习中
            BChapter bChapter = bChapterRepository.selectByPrimaryKey(queryChapterListResDTO.getPid());
            if (null != bChapter) {
                queryChapterListResDTO.setPSort(bChapter.getSort());
                queryChapterListResDTO.setProgress(Double.parseDouble(queryChapterListResDTO.getProgress()) * 100 + "%");
            }
            map.put("currentChapter", queryChapterListResDTO);
        } else if (StringUtils.isNotEmpty(chapterId)) {
            //外部分享章节
            //chapterId仅在外部分享章节时传
            List<QueryChapterListResDTO> queryChapterListResDTOList = bChapterRepository.getShareChapterById(chapterId);
            if (queryChapterListResDTOList != null && queryChapterListResDTOList.size() > 0 && queryChapterListResDTOList.get(0) != null) {
                queryChapterListResDTO = queryChapterListResDTOList.get(0);
                BChapter bChapter = bChapterRepository.selectByPrimaryKey(chapterId);
                if (bChapter != null && StringUtils.isNotEmpty(bChapter.getPid())) {
                    BChapter pChapter = bChapterRepository.selectByPrimaryKey(bChapter.getPid());
                    if (null != pChapter) {
                        queryChapterListResDTO.setPSort(pChapter.getSort());
                    }
                }
                map.put("currentChapter", queryChapterListResDTO);
            } else {
                map.put("currentChapter", "");
            }
        } else {
            //未加入我学习中或外部分享
            map.put("currentChapter", "");
        }

        return map;
    }

    /**
     * dto查询章节信息
     *
     * @param model
     * @return
     */
    @Override
    public List<QueryChapterListResDTO> queryChapterListResDTO(QueryChapterListDTO model, String corpId) {
        Map param = new HashMap<String, Object>();
        if (model.getLessonId() != null && !"".equals(model.getLessonId())) {
            param.put("lessonId", model.getLessonId());
        }
        if (StringUtils.isNotBlank(model.getChapterId())) {
            param.put("chapterId", model.getChapterId());
        }
        param.put("corpId", corpId);
        return bChapterRepository.queryBChapters(param);
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel persist(ChapterDTO chapterDTO, String corpId, uploadDTO dto) {
        if (chapterDTO.getChapterLevel() == 2 && "true".equals(replaceHttp)) {
            if (dto.getVideoUrl() != null) {
                LogUtil.fileChangeLog("----------替换oss链接https----------");
                dto.setVideoUrl(dto.getVideoUrl().replace("http:", "https:"));
            }
        }
        int line = 0;
        BFile bFile = new BFile();
        BChapter model = new BChapter();
        if (chapterDTO.getChapterLevel() == 2 && (dto.getFileSize() == null || "".equals(dto.getFileSize()))) {
            throw new ServiceException(ResultServiceEnums.FILE_NOT_NULL);
        }

        //将dto中的数据放入bchapter实体类中
        model.setBusinessId(chapterDTO.getBusinessId());
        model.setLessonId(chapterDTO.getLessonId());
        model.setChapterName(chapterDTO.getChapterName());
        model.setPid(chapterDTO.getPid());
        model.setChapterLevel(chapterDTO.getChapterLevel());
        model.setIsTest(chapterDTO.getIsTest());
        model.setState("1");

        bFile.setFileName(dto.getFileName());
        bFile.setVideoUrl(dto.getVideoUrl());
        bFile.setType(dto.getType());
        bFile.setFileSize(dto.getFileSize());

//        if (StringUtils.isNotEmpty(dto.getFileTime()) && dto.getFileTime().contains(".")) {
//            dto.setFileTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));;
//        }

        if (model.getBusinessId() == null) {
            //新建章节
            UserUtil.setCurrentPersistOperation(model);
            if (model.getClassHour() == null || model.getClassHour().equals("")) {
                model.setClassHour("");
            }
            if (StringUtils.isEmpty(model.getPid())) {
                model.setPid("-1");
            }
            if (model.getIsTest() == null || model.getIsTest().equals("")) {
                model.setIsTest("");
            }
            model.setIsTest(dto.getType());
            int sort = bChapterRepository.selectMaxSort(model.getPid(), model.getLessonId());
            model.setSort(sort);

            if (model.getChapterLevel() == 1) {
                chapterDTO.setBusinessId(model.getBusinessId());
                chapterDTO.setPid(model.getPid());
                //添加章
                line = bChapterRepository.insertSelective(model);
            } else {
                model.setClassHour("1");
//            model.setCorpId(corpId);
                bFile.setChapterId(model.getBusinessId());
                bFile.setLessonId(model.getLessonId());

                bFile.setFileTime(dto.getFileTime());
                model.setCourseTime(dto.getFileTime());
                if (StringUtils.isNotEmpty(dto.getFileTime()) && dto.getFileTime().contains(".")) {
                    bFile.setFileTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
                    model.setCourseTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
                }
                UserUtil.setCurrentPersistOperation(bFile);
                //新建节数据插入到chapter表
                chapterDTO.setIsTest(dto.getType());
                if ("3".equals(dto.getType()) && dto.getPdfNum() != null && !"".equals(dto.getPdfNum()) && Integer.parseInt(dto.getPdfNum()) > 1) {
                    //pdf 且页数大于每页限制
                    int num = Integer.parseInt(dto.getPdfNum());
                    List<BChapter> bChapterList = new ArrayList<>();
                    List<BFile> bFileList = new ArrayList<>();
                    int i = 1;
                    while (i <= num) {
                        BChapter newBchapter = new BChapter();
                        BFile newBfile = new BFile();
                        BeanUtils.copyProperties(bFile, newBfile);
                        BeanUtils.copyProperties(model, newBchapter);
                        if (i > 1) {
                            newBchapter.setCommonBusinessId();
                        }
                        newBchapter.setSort(model.getSort() + i - 1);
                        newBchapter.setChapterName(newBchapter.getChapterName() + i);
                        bChapterList.add(newBchapter);
                        newBfile.setCommonBusinessId();
                        newBfile.setChapterId(newBchapter.getBusinessId());
                        bFileList.add(newBfile);
                        i++;
                    }
                    for (BChapter b : bChapterList) {
                        line = bChapterRepository.insertSelective(b);
                        String cid = bChapterRepository.getFirstChapter(b.getLessonId());
                        if (cid.equals(b.getBusinessId())) {
                            bChapterRepository.updateVideoTimeInLesson(b.getCourseTime(), b.getLessonId());
                        }
                        bChapterRepository.updateClassHourInLesson(b.getLessonId());
                        this.updateLessonProgress(chapterDTO.getLessonId(), true, "");
                    }
                    //上传文件数据插入到bfile表中
                    bFileRepository.insertUploadFileList(bFileList);

                } else {
                    line = bChapterRepository.insertSelective(model);
                    //先判断该课程的第一节是否是本节  更新blesson表中course_time字段
                    String cid = bChapterRepository.getFirstChapter(model.getLessonId());
                    if (cid.equals(model.getBusinessId())) {
                        bChapterRepository.updateVideoTimeInLesson(model.getCourseTime(), model.getLessonId());
                    }

                    bChapterRepository.updateClassHourInLesson(model.getLessonId());//更新b_lesson表中 class_hour
                    // 新增节 才更新进度
                    if (2 == chapterDTO.getChapterLevel()) {
                        // 更新课程总进度
                        this.updateLessonProgress(chapterDTO.getLessonId(), true, "");
                    }
                    //上传文件数据插入到bfile表中
                    bFileRepository.insertUploadFile(bFile);
                }
            }
        } else {//修改
            if (model.getChapterLevel() == 1) {
                int x = bChapterRepository.updateChapterName(model.getChapterName(), model.getBusinessId());
                return new PersistModel(x);
            }
            BFile bFileOld = bFileRepository.selectInfoByChapterId(chapterDTO.getBusinessId());
            if (bFileOld.getVideoUrl().equals(dto.getVideoUrl())) {
                dto.setPdfNum("0");
            }
            if ("3".equals(dto.getType()) && dto.getPdfNum() != null && !"".equals(dto.getPdfNum()) && Integer.parseInt(dto.getPdfNum()) > 1) {
                model.setChapterName(model.getChapterName() + "-1");
            }
            UserUtil.setCurrentMergeOperation(model);
            bFile.setChapterId(model.getBusinessId());
            UserUtil.setCurrentMergeOperation(bFile);

            model.setCourseTime(dto.getFileTime());
            model.setIsTest(dto.getType());

            bFile.setFileTime(dto.getFileTime());
            if (StringUtils.isNotEmpty(dto.getFileTime()) && dto.getFileTime().contains(".")) {
                model.setCourseTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
                bFile.setFileTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
            }
            //先判断该课程的第一节是否是本节 更新blesson表中course_time字段
            String cid = bChapterRepository.getFirstChapter(model.getLessonId());
            if (model.getBusinessId().equals(cid)) {
                bChapterRepository.updateVideoTimeInLesson(model.getCourseTime(), model.getLessonId());
            }
            //判断修改前后pid是否发生改变
            BChapter bChapter = bChapterRepository.selectByPrimaryKey(model.getBusinessId());//获取当前节的信息
            if (bChapter != null && !bChapter.getPid().equals(model.getPid())) {
                int sort = bChapterRepository.selectMaxSort(model.getPid(), model.getLessonId());
                model.setSort(sort);
                List<BChapter> cutPdfChapterList = bChapterRepository.getPdfIdList(model.getBusinessId());
                int i = 1;
                if (cutPdfChapterList != null && cutPdfChapterList.size() > 1) {
                    for (BChapter b : cutPdfChapterList) {
                        if (b.getBusinessId().equals(model.getBusinessId())) {
                            b.setChapterName(model.getChapterName());
                        }
                        b.setPid(model.getPid());
                        b.setSort(sort + i++);
                        line = bChapterRepository.updateByPrimaryKeySelective(b);
                    }
                } else {
                    line = bChapterRepository.updateByPrimaryKeySelective(model);
                }
                //更新修改之前 章中节的sort
                reSort(bChapter.getPid(), bChapter.getLessonId());
                //更新修改之后 章中节的sort
                reSort(model.getPid(), model.getLessonId());
            } else {
                line = bChapterRepository.updateByPrimaryKeySelective(model);
            }

            if (model.getChapterLevel() == 1) {
                return new PersistModel(line);
            }
            bFileRepository.updateUploadFile(bFile);

            //修改资料为 pdf 且页数大于切割数量
            if ("3".equals(dto.getType()) && dto.getPdfNum() != null && !"".equals(dto.getPdfNum()) && Integer.parseInt(dto.getPdfNum()) > 1) {
                //pdf 且页数大于每页限制
                int num = Integer.parseInt(dto.getPdfNum());
                List<BChapter> bChapterList = new ArrayList<>();
                List<BFile> bFileList = new ArrayList<>();
                BChapter chapter = bChapterRepository.selectByPrimaryKey(model.getBusinessId());
                List<String> sortList = bChapterRepository.getPDFSortList(model.getBusinessId(), model.getPid());
                int i = 2;
                while (i <= num) {
                    BChapter newBchapter = new BChapter();
                    BFile newBfile = new BFile();
                    BeanUtils.copyProperties(bFile, newBfile);
                    BeanUtils.copyProperties(chapter, newBchapter);
                    newBchapter.setCommonBusinessId();
                    newBchapter.setSort(chapter.getSort() + i - 1);
                    newBchapter.setChapterName(newBchapter.getChapterName().substring(0, newBchapter.getChapterName().lastIndexOf("-")) + "-" + i);
                    bChapterList.add(newBchapter);
                    UserUtil.setCurrentPersistOperation(newBfile);
                    newBfile.setCommonBusinessId();
                    newBfile.setLessonId(chapter.getLessonId());
                    newBfile.setFileTime(dto.getFileTime());
                    newBfile.setChapterId(newBchapter.getBusinessId());
                    bFileList.add(newBfile);
                    i++;
                }
                for (BChapter b : bChapterList) {
                    line = bChapterRepository.insertSelective(b);
                    bChapterRepository.updateClassHourInLesson(b.getLessonId());
                    this.updateLessonProgress(chapterDTO.getLessonId(), true, "");
                }
                //上传文件数据插入到bfile表中
                bFileRepository.insertUploadFileList(bFileList);
                if (sortList.size() > 0) {
                    int zx = bChapterRepository.updatePdfSort(sortList, num - 1);
                }
            }

            // 判断 新文件 更新 课程学习进度（总进度 此章节进度）
            if (bFileOld != null && StringUtils.isNotEmpty(bFileOld.getVideoUrl()) && !(bFileOld.getVideoUrl().equals(dto.getVideoUrl()))) {
                // 所有学习过此课程的 学生课程信息
                List<BLessonPerson> bLessonPersonList = bLessonPersonRepository.selectByLessonId(chapterDTO.getLessonId(), "");
                BLesson bLesson = bLessonRepository.selectByPrimaryKey(chapterDTO.getLessonId());
                NumberFormat numberFormat = NumberFormat.getInstance();
                numberFormat.setMaximumFractionDigits(2);
                for (BLessonPerson lessonPerson : bLessonPersonList) {
                    // 更新 学生针对此章节的中间表
                    List<BRStudentChapter> brStudentChapterList = brStudentChapterRepository.queryByStuIdAndChapter(chapterDTO.getBusinessId(), "");
                    for (BRStudentChapter studentChapter : brStudentChapterList) {
                        if (lessonPerson.getPersonId().equals(studentChapter.getStudentId())) {

                            double learnProgress = Double.parseDouble(lessonPerson.getLearnProgress().split("%")[0]) / 100;
                            double newLearnProgress = Double.parseDouble(numberFormat.format((float) (learnProgress * Integer.parseInt(bLesson.getClassHour()) - studentChapter.getProgress()) / Float.parseFloat(bLesson.getClassHour())));
                            if (newLearnProgress == 0) {
                                lessonPerson.setIsFinish("0");
//                                lessonPerson.setFinishDate(null);
                                lessonPerson.setFinishDateSetNull("1");
                            }
                            if (0 < newLearnProgress && newLearnProgress < 1) {
                                lessonPerson.setIsFinish("1");
//                                lessonPerson.setFinishDate(null);
                                lessonPerson.setFinishDateSetNull("1");
                            }
                            // 规避 复数 和超100% 的情况
                            newLearnProgress = newLearnProgress > 1 ? 1 : newLearnProgress < 0 ? 0 : newLearnProgress;

                            lessonPerson.setLearnProgress(newLearnProgress * 100 + "%");
                            bLessonPersonRepository.updateAtChapter(lessonPerson);
                            studentChapter.setProgress(0);
                            studentChapter.setCurrentLocation(0);
                            brStudentChapterRepository.updateById(studentChapter);
                        }
                    }
                }
            }
        }

        //不是为添加章时插入资料表
        if (!"0".equals(dto.getFileSize()) && StringUtils.isNotEmpty(model.getIsTest())) {
            BMaterial bMaterial = new BMaterial();
            UserUtil.setCurrentPersistOperation(bMaterial);
            bMaterial.setLessonId(chapterDTO.getLessonId());
            bMaterial.setChapterId(model.getBusinessId());
            bMaterial.setFileName(dto.getFileName());

            if (StringUtils.isNotEmpty(dto.getFileTime()) && dto.getFileTime().contains(".")) {
                bMaterial.setFileTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
            }

            bMaterial.setFileUrl(dto.getVideoUrl());

            bMaterial.setType(model.getIsTest());
            bMaterial.setUploadType("1");
            bMaterial.setFileSize(dto.getFileSize());
            bMaterialRepository.insertMaterial(bMaterial);
        }
        //查询 当前资源共享状态
        String flag = bMessageRepository.getSycnValue(model.getCorpId());

        //判断 该章节对应课程 的发布状态 （已发布和已关闭才执行以下操作）
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(model.getLessonId());
        if (ReleaseStateEnum.RELEASED.getCode().equals(bLesson.getReleaseState())
                || ReleaseStateEnum.CLOSED.getCode().equals(bLesson.getReleaseState())) {
            // 保存日志
            bLessonLogRepository.insertSelective(BLessonLog.generate("修改课程", model.getLessonId()));
            //开启共享状态 编辑课程时 同步
            if (LessonSycnEnum.OPEN.getCode().equals(flag)) {
                // 同步 资源共享 更新 缓存章节 文件 两个表（未发布的课程 之前不会被同步到缓存表中）
                List<BChapter> chapters = new ArrayList<>();
                List<BFile> files = new ArrayList<>();
                //step2 根据课程查询对应的章节和课件
                if (DelStatus.DELETED.getStatus().equals(bLesson.getDelFlag())) {
                    bLesson.setDelFlag(DelStatus.NORMAL.getStatus());
                    bLesson.setReleaseState(ReleaseStateEnum.DELETE.getCode());
                }
                List<BChapter> chapterList = bChapterRepository.queryBChaptersByLessonId(bLesson.getBusinessId());
                List<BFile> fileList = bFileRepository.getFileByLessonId(bLesson.getBusinessId());
                chapters.addAll(chapterList);
                files.addAll(fileList);
                //step3 将数据插入 缓存 章节、课件 表(全删全加)
                bpChapterRepository.deleteList(chapters);
                bpFileRepository.deleteList(files);
                bpChapterRepository.insertList(chapters);
                bpFileRepository.insertList(files);
            }
        }
        return new PersistModel(line);
    }

    /**
     * @param lessonId
     * @param add      true: 新增 false 删除
     */
    // 新增章节 - 更新学生观看课程总进度
    private void updateLessonProgress(String lessonId, boolean add, String chapterId) {
        NumberFormat numberFormat = NumberFormat.getInstance();
        numberFormat.setMaximumFractionDigits(2);
        List<BLessonPerson> bLessonPersonList = bLessonPersonRepository.selectByLessonId(lessonId, "");
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(lessonId);


        for (BLessonPerson lessonPerson : bLessonPersonList) {
            if (!add && null != lessonPerson.getChapterId() && lessonPerson.getChapterId().equals(chapterId)) {
                bLessonPersonRepository.clearChapterId(lessonPerson.getBusinessId());
            }
            String newLearnProgress;
            if (lessonPerson.getLearnProgress() == null) {
                break;
            }
            List<BRStudentChapter> brStudentChapterList = brStudentChapterRepository.queryByStuIdAndChapter(chapterId, lessonPerson.getPersonId());
            double oldProgress = 0;

            if (null != brStudentChapterList && brStudentChapterList.size() > 0) {
                oldProgress = brStudentChapterList.get(0).getProgress();
            }
            double learnProgress = Double.parseDouble(lessonPerson.getLearnProgress().split("%")[0]) / 100;
            if (add) {
                newLearnProgress = Integer.parseInt(bLesson.getClassHour()) - 1 == 0 ? "0" : numberFormat.format(learnProgress * (Integer.parseInt(bLesson.getClassHour()) - 1) / (Double.parseDouble(bLesson.getClassHour())));
            } else {
                int oldClassHour = Integer.parseInt(bLesson.getClassHour()) + 1;
                newLearnProgress = numberFormat.format((learnProgress * oldClassHour - oldProgress) / (oldClassHour - 1));
            }
            if (0 == Double.parseDouble(newLearnProgress)) {
                lessonPerson.setIsFinish("0");
//                lessonPerson.setFinishDate(null);
                lessonPerson.setFinishDateSetNull("1");
            } else {
                if (learnProgress > 0 && Double.parseDouble(newLearnProgress) < 0.01) {
                    newLearnProgress = "0.01";
                }
                if (IsFinishEnum.FINISHED.getCode().equals(lessonPerson.getIsFinish()) && Double.parseDouble(newLearnProgress) < 1) {
                    lessonPerson.setIsFinish("1");
//                    lessonPerson.setFinishDate(null);
                    lessonPerson.setFinishDateSetNull("1");
                }
                if (Double.parseDouble(newLearnProgress) >= 1) {
                    newLearnProgress = "1";
                    lessonPerson.setIsFinish("2");
                }
            }

            //LZ 判断 曾经是否学完过 奖励积分
            if (IsFinishEnum.FINISHED.getCode().equals(lessonPerson.getIsFinish()) && !(IsFinishEnum.LEARNING.getCode().equals(lessonPerson.getEverFinished()))) {
                //查询课程信息
                if (StringUtils.isNotEmpty(bLesson.getValueGain()) && Long.parseLong(bLesson.getValueGain()) > 0) {
                    BPersonValue bPersonValue = new BPersonValue(lessonPerson.getPersonId(), "学完!@#课程", bLesson.getValueGain(), "0", bLesson.getBusinessId());
                    bLessonServiceImpl.addValueEvent(lessonPerson.getPersonId(), bPersonValue, null);
                }
                lessonPerson.setEverFinished("1");
            }
            // 规避 复数 和超100% 的情况
            float tempTotalProgress = Float.parseFloat(newLearnProgress);
            newLearnProgress = String.valueOf(tempTotalProgress > 1 ? 1 : tempTotalProgress < 0 ? 0 : tempTotalProgress);

            lessonPerson.setLearnProgress(Float.parseFloat(newLearnProgress) * 100 + "%");
            bLessonPersonRepository.updateAtChapter(lessonPerson);
            // 根据chapter_id 删除b_r_student_chapter
            brStudentChapterRepository.deleteBatchByChapterId(chapterId);

        }
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel update(BChapter model) {
        int line = bChapterRepository.updateByPrimaryKeySelective(model);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel removeBChapter(String oldId, CurUser curUser) {
        BChapter bChapter = bChapterRepository.selectByPrimaryKey(oldId);
        String id = oldId;
        if (bChapter.getChapterLevel() != 1) {
            //根据章节id 获取与该节oss地址相同的第一个节id
            id = bChapterRepository.getFirstOssId(oldId);
            bChapter = bChapterRepository.selectByPrimaryKey(id);
        }
        String lessonId = bChapterRepository.getLessonIdBychaperId(id);
        if (bChapter == null)
            throw new ServiceException(ResultServiceEnums.CHAPTER_NOT_EXISTS);

        int chapterCount = bChapterRepository.getlessonHours(lessonId);
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(lessonId);
        if ((ReleaseStateEnum.RELEASED.getCode().equals(bLesson.getReleaseState()) || ReleaseStateEnum.CLOSED.getCode().equals(bLesson.getReleaseState())) && chapterCount == 1 && bChapter.getChapterLevel() != 1) {
            throw new ServiceException(ResultServiceEnums.CHAPTER_ONLY_ONE);
        }
        List<QueryChapterListResDTO> childList = bChapterRepository.queryBChaptersByPid(id);
        boolean hasChild = childList != null && childList.size() > 0;
        boolean isChapter = bChapter.getChapterLevel() == ChapterLevelEnum.CHAPTER.getCode();
        if (isChapter && hasChild) {
            throw new ServiceException(ResultServiceEnums.CHAPTER_HAS_CHILD);
        }
        if (isChapter) {
            int line = bChapterRepository.removeChapter(id);
            return new PersistModel(line);
        }
        //获取第一节课的id 与当前id对比
        String firstId = bChapterRepository.getFirstChapter(lessonId);
        int line = bChapterRepository.removeByPrimaryKey(id);
        bChapterRepository.updateClassHourInLesson(lessonId);
        if (StringUtils.isNotEmpty(id) && id.equals(firstId)) {
            //当前节为第一节 获取下一节的课程时长
            String secondId = bChapterRepository.getFirstChapter(lessonId);
            String time = bChapterRepository.getTimeByChapterId(secondId);
            //更新到b_lesson 表
            bChapterRepository.updateCourseTimeForLesson(time, lessonId);
        }
        //获取删除的课的id 与当前用户最近观看课的id对比
//        QueryChapterListResDTO queryChapterListResDTO = bChapterRepository.getWithCurrentChapterBylessonId(lessonId, curUser.getUserId(), curUser.getCorpId());
//        String currentId =  bChapterRepository.getFirstChapter(queryChapterListResDTO.getBusinessId());
//        if(StringUtils.isNotEmpty(id) && id.equals(currentId)){
//            BLessonPerson bLessonPerson = new BLessonPerson();
//            bLessonPerson.setBusinessId(queryChapterListResDTO.getBLPBusinessId());
//            bLessonPerson.setChapterId(null);
//            bLessonPerson.setLearnProgress(null);
////            bLessonPerson.setIsFinish("");
//            // 删除 b_r_student_chapter
//            brStudentChapterRepository.deleteByPrimaryKey(queryChapterListResDTO.getBRSBusinessId());
//        }
        //更新表中sort
        reSort(bChapter.getPid(), bChapter.getLessonId());
//        List<BRStudentChapter> brStudentChapterList = brStudentChapterRepository.queryByStuIdAndChapter(id, curUser.getUserId());

        // 判断 2-节 才重新计算进度
        if (2 == bChapter.getChapterLevel()) {
//            double oldProgress = 0;
//            if (null !=brStudentChapterList && brStudentChapterList.size()>0){
//                oldProgress = brStudentChapterList.get(0).getProgress();
//            }
            // 根据chapter_id 删除b_r_student_chapter
//            brStudentChapterRepository.deleteBatchByChapterId(id);
            // 更新学生 对应章节总进度
            try {
                this.updateLessonProgress(lessonId, false, id);
            } catch (Exception e) {
                throw new ServiceException(ResultServiceEnums.CHAPTER_ONLY_ONES);
            }
        }

        //删除student_chapter表中数据
        bChapterRepository.deleteBStudentChapter(id, curUser.getUserId());

        //查询 当前资源共享状态
        String flag = bMessageRepository.getSycnValue(curUser.getCorpId());
        //开启共享状态 编辑课程时 添加日志
        if (LessonSycnEnum.OPEN.getCode().equals(flag)) {
            bLessonLogRepository.insertSelective(BLessonLog.generate("删除章节", oldId));
        }

        return new PersistModel(line);
    }

    @Override
    public String changeFileToPdf(String chapterId) {
        if (StringUtils.isEmpty(chapterId)) {
            return null;
        }
        //1.  获取文件信息
        BFile bFile = bFileRepository.selectInfoByChapterId(chapterId);
        return changeFile(bFile).getUrl();
    }

    @Override
    public FileToPdfAtViewDTO changeFileToPdfAtView(BFile bFile) {
        if (StringUtils.isEmpty(bFile.getVideoUrl()) || StringUtils.isEmpty(bFile.getLessonId()) || StringUtils.isEmpty(bFile.getType())) {
            return null;
//            throw new ServiceException(ResultServiceEnums.PARAM_NOT_NULL);
        }
        LogUtil.fileChangeLog("----------文件下载及转换----------");
        return changeFile(bFile);
    }

    @Override
    public String getDownLoadUrl(BFile bFile) {
        if (StringUtils.isEmpty(bFile.getVideoUrl()) || StringUtils.isEmpty(bFile.getLessonId())) {
            return null;
//            throw new ServiceException(ResultServiceEnums.PARAM_NOT_NULL);
        }
        String name = bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);
        String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bFile.getLessonId();
        String returnBasePath = global.getRETURN_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bFile.getLessonId();
        File file = new File(filePath);
        File fileOld = new File(filePath + File.separator + name);
        if (fileOld.exists()) {
            return returnBasePath + File.separator + name;
        } else {
            String downLoadName = downLoadFile(bFile, file, filePath, "0");
            return returnBasePath + File.separator + downLoadName;
        }
    }

    @Override
    public String getBatchDownload(String bFileJson) {
        //批量打包下载
        List<BFile> fileList = JSONObject.parseArray(bFileJson, BFile.class);
        if (fileList == null || fileList.size() < 1) {
            throw new ServiceException(ResultServiceEnums.NOT_DOWNLOAD_FILE);
        }
        //判断文件是否存在  不存在下载  并将这些文件包装成File
        fileList.forEach(bFile -> {
            if (StringUtils.isEmpty(bFile.getVideoUrl()) || StringUtils.isEmpty(bFile.getLessonId())) {
                return;
            }
            String name = bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);
            String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bFile.getLessonId();
            File file = new File(filePath);
            File fileOld = new File(filePath + File.separator + name);
            if (fileOld.exists()) {
                bFile.setFileUrl(filePath + File.separator + name);
            } else {
                String downLoadName = downLoadFile(bFile, file, filePath, "0");
                bFile.setFileUrl(filePath + File.separator + downLoadName);
            }
        });
        if (fileList.size() < 1) {
            throw new ServiceException(ResultServiceEnums.NOT_DOWNLOAD_FILE);
        }
        String uuid = IdGen.uuid();
        //文件打包地址
        File zipFile = new File(global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + global.getZIP_LOCATION() + File.separator + uuid + ".zip");
        log.info("---------------批量下载----------zip压缩包 相对路径--------" + zipFile.getPath());
        //返回地址
        String returnAddress = global.getRETURN_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + global.getZIP_LOCATION() + File.separator + uuid + ".zip";
        log.info("---------------批量下载----------zip压缩包 相对路径--------" + returnAddress);
        //打包
        ZipMultiFile.zipFiles(fileList, zipFile);
        return returnAddress;
    }

    @Override
    public String getBatchsDownload(String ids) {
        // 拆分ids
        String[] lessonIds = ids.split(",");
        String batchUDID = "批量下载打包" + System.currentTimeMillis();
        //遍历每个id
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        for (String id : lessonIds) {
            List<BFile> fileList = bFileRepository.getFileByLessonId(id);
            //batch是zip打包文件夹     bFile.getLessonId()替换形成name
            BLesson bLesson = bLessonRepository.selectByPrimaryKey(id);
            String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + batchUDID + File.separator + bLesson.getLessonName() + "-" + simpleDateFormat.format(bLesson.getCreateDate());
            File file = new File(filePath);
            if (fileList == null || fileList.size() < 1)
                continue;
            //下载文件  并将这些文件包装成File
            fileList.forEach(bFile -> {
                if (StringUtils.isNotEmpty(bFile.getVideoUrl()))
                    downLoadFile(bFile, file, filePath, "1");
            });
        }
        String uuid = IdGen.uuid();
        //文件打包地址
        File zipFilePath = new File(global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + global.getZIP_LOCATION());
        if (!zipFilePath.exists())
            zipFilePath.mkdirs();
        //定义压缩包的生成路径
        File zipFile = new File(zipFilePath.getPath() + File.separator + uuid + ".zip");
        log.info("---------------批量课程下载----------zip压缩包 相对路径--------" + zipFile.getPath());
        //返回地址
        String returnAddress = global.getRETURN_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + global.getZIP_LOCATION() + File.separator + uuid + ".zip";
        log.info("---------------批量课程下载----------zip压缩包 网络访问路径--------" + returnAddress);
        try {
            String zipUrl = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + batchUDID;
            //打包
            ZipMultiFile.toZip(zipUrl, new FileOutputStream(zipFile), true);
            //删除 文件夹
            ZipMultiFile.deleteDirectory(new File(zipUrl));

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return returnAddress;
    }


    @Override
    public String getLessonDownload(String id) {
        //获取当前课程的信息
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(id);
        List<BFile> fileList = bFileRepository.getFileByLessonId(id);
        //batch是zip打包文件夹     bFile.getLessonId()替换形成name
        String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bLesson.getLessonName() ;
        File file = new File(filePath);
        if (!file.exists())
            file.mkdirs();
        if (fileList != null && fileList.size() > 0){
            //下载文件  并将这些文件包装成File
            fileList.forEach(bFile -> {
                if (StringUtils.isNotEmpty(bFile.getVideoUrl()))
                    downLoadFile(bFile, file, filePath, "1");
            });
        }
        String uuid = IdGen.uuid();
        //文件打包地址
        File zipFilePath = new File(global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + global.getZIP_LOCATION());
        if (!zipFilePath.exists())
            zipFilePath.mkdirs();
        //定义压缩包的生成路径
        File zipFile = new File(zipFilePath.getPath() + File.separator + uuid + ".zip");
        log.info("---------------批量课程下载----------zip压缩包 相对路径--------" + zipFile.getPath());
        //返回地址
        String returnAddress = global.getRETURN_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + global.getZIP_LOCATION() + File.separator + uuid + ".zip";
        log.info("---------------批量课程下载----------zip压缩包 网络访问路径--------" + returnAddress);
        try {
            //打包
            ZipMultiFile.toZip(filePath, new FileOutputStream(zipFile), true);
            //删除 文件夹
            ZipMultiFile.deleteDirectory(file);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return returnAddress;
    }

    @Override
    public String fileReadable(BFile bFile) {
        String flag = "1";
        String name = bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);
        String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + "temp";
        File file = new File(filePath);
        File fileOld = new File(filePath + File.separator + name);
        String path;
        if (fileOld.exists()) {
            path = fileOld.getPath();
        } else {
            LogUtil.fileChangeLog("------------------下载-------------------------" + fileOld);
            String downLoadName = downLoadFile(bFile, file, filePath, "0");
            LogUtil.fileChangeLog("------------------判断可读-------------------------" + fileOld);
            path = filePath + File.separator + downLoadName;
        }
        if ("7".equals(bFile.getType()) || "8".equals(bFile.getType()) || "9".equals(bFile.getType()) || "10".equals(bFile.getType())) {
            FileReadableUtil.imgReadable(path);
        } else {
            FileReadableUtil.fileReadable(path, fileMaxSize, pdfMaxSize, txtMaxSize, fileMaxPages);
        }
        return flag;
    }

    @Override
    public String checkFileHeader(BFile bFile) {
        String name = bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);
        String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + "temp";
        File file = new File(filePath);
        File fileOld = new File(filePath + File.separator + name);
        if (fileOld.exists()) {
            LogUtil.fileChangeLog("------------------文件名重复-------------------------" + fileOld);
            throw new ServiceException(ResultServiceEnums.DUP_FILE);
        } else {
            LogUtil.fileChangeLog("------------------下载-------------------------" + fileOld);
            String downLoadName = downLoadFile(bFile, file, filePath, "0");
            LogUtil.fileChangeLog("------------------判断后缀名相符-------------------------" + fileOld);
            Boolean line = CheckFileHeaderUtil.toCheck(filePath + File.separator + downLoadName);
            if (!line) {
                throw new ServiceException(ResultServiceEnums.FILE_CHECK_ERROR);
            }
            LogUtil.fileChangeLog("-----------------------后缀名相符-------------------------" + fileOld);
        }
        return "1";

    }

    @Override
    public int checkHeaderFromFile(MultipartFile file) {
        if (file == null)
            throw new ServiceException(ResultServiceEnums.NOT_FILE);

        if (!CheckFileHeaderUtil.toCheckFile(file))
            throw new ServiceException(ResultServiceEnums.FILE_CHECK_ERROR);

        return 1;
    }


    public FileToPdfAtViewDTO changeFile(BFile bFile) {
        FileToPdfAtViewDTO dto = new FileToPdfAtViewDTO();
        List<String> pdfOssUrls = new ArrayList<>();
        LogUtil.fileChangeLog("****************request + begin***********" + DateFormatUtils.format(new Date(), "HH:mm:ss"));
        String name = bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);
        String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bFile.getLessonId();
        String returnBasePath = global.getRETURN_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bFile.getLessonId();
        File file = new File(filePath);
        //2. pdf 本地是否存在， word ppt 转换后的pdf是否存在
        LogUtil.fileChangeLog("------------------获取原路径文件-----------------------------" + filePath);
        File fileOld = new File(filePath + File.separator + name);
        LogUtil.fileChangeLog("------------------原文件路径-----------------------------" + fileOld);
        String newName = name.substring(0, name.lastIndexOf(".")) + "-pdf.pdf";
        LogUtil.fileChangeLog("------------------新文件名-----------------------------" + newName);
        //转换后的路径
        File fileNew = new File(filePath + File.separator + newName);
        LogUtil.fileChangeLog("------------------判断文件类型-----------------------------" + fileNew.getPath());
        if (FileTypeEnum.PDF.getCode().equals(bFile.getType())) {
            String localPath = "";
            if (fileOld.exists()) {
                localPath = filePath + File.separator + name;
            } else {
                String downLoadName = downLoadFile(bFile, file, filePath, "0");
                localPath = filePath + File.separator + downLoadName;
            }
            int pages = FileReadableUtil.pdfReadable(localPath);
            /**
             * 文件已存在或已下载    服务器相对文件路径 localPath  和   pdf页数 pages
             */
            if (pages > pdfCutPage) {
                if (bFile.getChapterId() == null || "".equals(bFile.getChapterId())) {
                    //第一次 预览
                    try {
                        List<String> pdfUrls = CutPdfUtil.toCutPdf(filePath + File.separator + name, name, pdfCutPage);
                        for (String url : pdfUrls) {
                            pdfOssUrls.add(url);
                        }
                        //返回拆分文件第一个路径
                        dto.setUrl(pdfUrls.get(0).replace(filePath, returnBasePath));
                        dto.setPdfNum(pdfUrls.size());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                } else {
                    //step1  根据章节id获取对应  文件的序号 x
                    List<BChapter> chapterIds = bChapterRepository.getPdfIdList(bFile.getChapterId());
                    int x = 0;
                    for (BChapter b : chapterIds) {
                        x++;
                        if (b.getBusinessId().equals(bFile.getChapterId())) {
                            break;
                        }
                    }
                    String cutPdf = filePath + File.separator + "pdf-" + name.substring(0, name.lastIndexOf(".")) +
                            File.separator + name.substring(0, name.lastIndexOf(".")) + "-" + x + ".pdf";
                    File pdf = new File(cutPdf);
                    //step2   判断文件是否已存在   存在（直接返回）  不存在（拆分后返回）
                    if (pdf.exists()) {
                        dto.setUrl(returnBasePath + File.separator + "pdf-" + name.substring(0, name.lastIndexOf(".")) +
                                File.separator + name.substring(0, name.lastIndexOf(".")) + "-" + x + ".pdf");
                    } else {
                        try {
                            List<String> pdfUrls = CutPdfUtil.toCutPdf(filePath + File.separator + name, name, pdfCutPage);
                            for (String url : pdfUrls) {
                                pdfOssUrls.add(url);
                            }
                            dto.setUrl(returnBasePath + File.separator + "pdf-" + name.substring(0, name.lastIndexOf(".")) +
                                    File.separator + name.substring(0, name.lastIndexOf(".")) + "-" + x + ".pdf");
                            dto.setPdfNum(pdfUrls.size());

                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }
            } else {
                //pdf页数少不满足切割条件 直接返回服务器绝对地址
                dto.setUrl(returnBasePath + File.separator + name);
            }
            return dto;


            /** 不下载pdf、文件转发方案（上传快，没有可读判断）
             String hostAddress = "";
             try{
             InetAddress address = InetAddress.getLocalHost();
             hostAddress = address.getHostAddress();
             }catch (UnknownHostException e){
             e.printStackTrace();
             }
             Long size = FileReadableUtil.getFileSize(bFile.getVideoUrl());
             if (size > (long)(1024 * 1024 * pdfMaxSize)){
             throw new ServiceException(400,"pdf文件超过最大"+pdfMaxSize+"M限制");
             }
             String path = hostAddress+"/eduFiles/education-dev/" + bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);
             return path;
             */
        }
        //3. pdf不存在，下载  ，  删除word ppt，重新下载，转换
        if (FileTypeEnum.PPT.getCode().equals(bFile.getType()) || FileTypeEnum.WORD.getCode().equals(bFile.getType()) || FileTypeEnum.TXT.getCode().equals(bFile.getType())) {
            LogUtil.fileChangeLog("------------------文件为ppt或word或txt----------------------------");
            if (fileNew.exists()) {
                LogUtil.fileChangeLog("------------------转换后文件-存在-直接返回-----------------------------");
                pdfOssUrls.add(bFile.getVideoUrl());
                dto.setUrl(returnBasePath + File.separator + newName);
                return dto;
            } else {
                LogUtil.fileChangeLog("------------------删除老文件----------------------------");
                fileOld.delete();
                LogUtil.fileChangeLog("------------------设置下载路径----------------------------");
                String downLoadFileName = downLoadFile(bFile, file, filePath, "0");

                //判断文件是否可读
                LogUtil.fileChangeLog("------------------判断文件是否可读-----------------------------" + filePath + File.separator + name);
                FileReadableUtil.fileReadable(filePath + File.separator + downLoadFileName, fileMaxSize, pdfMaxSize, txtMaxSize, fileMaxPages);
                LogUtil.fileChangeLog("------------------------非pdf文件可读-----------------------" + downLoadFileName);
                LogUtil.fileChangeLog("-----------------------------------------------downLoadFileUrl：" + filePath + File.separator + downLoadFileName);
                LogUtil.fileChangeLog("-----------------------------------------------filePath：" + filePath + File.separator + newName);
                int result = officeToPdf.transformToPdf(filePath + File.separator + downLoadFileName, filePath + File.separator + newName);
                LogUtil.fileChangeLog("------------------转换完毕-----------------------------");
                if (result < 1) {
                    log.error("文件转换异常");
                    throw new ServiceException(ResultServiceEnums.CHANGE_FILE_ERROR);
                }
                //4. 更新在 bfile 表    ， ppt和word  存一下转换之后的url filePath
                if (StringUtils.isNotEmpty(bFile.getChapterId())) {
                    LogUtil.fileChangeLog("------------------章节存在-----------------------------");
                    bFile.setFileUrl(filePath + File.separator + newName);
                    LogUtil.fileChangeLog("------------------存转换后文件路径-filePath-----------------------------");
                    bFileRepository.updateByPrimaryKeySelective(bFile);
                }
                LogUtil.fileChangeLog("****************request + end***********" + DateFormatUtils.format(new Date(), "HH:mm:ss"));

                pdfOssUrls.add(bFile.getVideoUrl());
                dto.setUrl(returnBasePath + File.separator + newName);
                return dto;
            }
        } else {
            return null;
        }
    }


    /**
     * 将阿里云文件下载到服务器本地
     *
     * @param bFile
     * @param file
     * @param filePath
     * @param trueName 0返回url后缀   1返回具体文件名
     * @return
     */
    public String downLoadFile(BFile bFile, File file, String filePath, String trueName) {
        if ("true".equals(replaceHttp)) {
            LogUtil.fileChangeLog("----------替换oss链接https----------");
            bFile.setVideoUrl(bFile.getVideoUrl().replace("http:", "https:"));
        }
        LogUtil.fileChangeLog("****************download + begin***********" + DateFormatUtils.format(new Date(), "HH:mm:ss"));
        String name = "";
        if ("1".equals(trueName)) {
            name = bFile.getFileName();
        } else {
            name = bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);
        }

        LogUtil.fileChangeLog("-----------------判断文件夹是否存在----------------------" + filePath);
        //判断文件夹是否存在
        if (!file.exists()) {
            LogUtil.fileChangeLog("------------------文件夹不存在，创建新的的文件夹-----------------------------");
            //如果文件夹不存在，则创建新的的文件夹
            file.mkdirs();
        }
        FileOutputStream fileOut = null;
        HttpURLConnection conn = null;
        InputStream inputStream = null;
        try {
            // 建立链接
            URL httpUrl = new URL(bFile.getVideoUrl());
            LogUtil.fileChangeLog("-----------------建立链接-----------------------------");
            conn = (HttpURLConnection) httpUrl.openConnection();
            LogUtil.fileChangeLog("------------------设置连接参数-----------------------------");
            //以Post方式提交表单，默认get方式
            conn.setRequestMethod("GET");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            //设置页面的字符编码
            conn.setRequestProperty("Charset", "UTF-8");
            LogUtil.fileChangeLog("------------------不使用缓存-----------------------------");
            // post方式不能使用缓存
            conn.setUseCaches(false);
            LogUtil.fileChangeLog("------------------连接指定的资源---------------------------");
            //连接指定的资源
            conn.connect();
            //获取网络输入流
            inputStream = conn.getInputStream();
            LogUtil.fileChangeLog("------------------获取网络输入流------------------------");
            BufferedInputStream bis = new BufferedInputStream(inputStream);
            LogUtil.fileChangeLog("------------------判断文件的保存路径后面是否以/结尾-----------------------------");
            //判断文件的保存路径后面是否以/结尾
            if (!filePath.endsWith(File.separator)) {
                LogUtil.fileChangeLog("------------------不以/结尾，拼接/-----------------------------" + filePath);
                filePath += File.separator;
            }
            //写入到文件（注意文件保存路径的后面一定要加上文件的名称）
            fileOut = new FileOutputStream(filePath + name);
            LogUtil.fileChangeLog("------------------写入到文件-----------------------------");
            BufferedOutputStream bos = new BufferedOutputStream(fileOut);
            byte[] buf = new byte[1024 * 512];
            int length = bis.read(buf);
            //保存文件
            LogUtil.fileChangeLog("------------------保存文件开始-----------------------------");
            while (length != -1) {
                LogUtil.fileChangeLog("------------------正在下载----------------------" + DateFormatUtils.format(new Date(), "HH:mm:ss"));
                bos.write(buf, 0, length);
                length = bis.read(buf);
            }
            LogUtil.fileChangeLog("------------------关闭--BufferedInputStream--和--BufferedOutputStream--流---------------------------");
            bos.close();
            bis.close();
            LogUtil.fileChangeLog("------------------断开--HttpURLConnection--连接---------------------------");
            conn.disconnect();
            LogUtil.fileChangeLog("****************download + end***********" + DateFormatUtils.format(new Date(), "HH:mm:ss"));
        } catch (Exception e) {
            log.error(e.getMessage() + "下载文件异常");
            throw new ServiceException(ResultServiceEnums.DOWNLOAD_ERROR);
        }
        LogUtil.fileChangeLog("------------------返回name--------------------------" + name);
        return name;
    }

    @Override
    @Transactional
    public void markScore(List<ScoreInfoDTO> scoreInfoDTOS) {
        for (ScoreInfoDTO scoreInfoDTO : scoreInfoDTOS) {
            String id = bChapterRepository.selectIdByStudentIdAndChapterId(scoreInfoDTO.getChapterId(), scoreInfoDTO.getStudentId());
            if (scoreInfoDTO.getScore() == null || scoreInfoDTO.getScore() < 0 || scoreInfoDTO.getScore() > 100)  //分数不为空且在0到100之间
                throw new ServiceException(ResultServiceEnums.SCORE_ERROR);
            if (id == null) {   //查看该学生该章节是否已经完成学习
                throw new ServiceException(ResultServiceEnums.UN_COMPLETE);
            } else {   //已经学习则修改
                scoreInfoDTO.setBusinessId(id);
                scoreInfoDTO.setAutomatic("0");
                UserUtil.setCurrentMergeOperation(scoreInfoDTO);
                bChapterRepository.updateScoreInfo(scoreInfoDTO);
            }
        }
    }

    @Override
    public List<StuScoreDTO> queryScoreListBySlIdAndStuId(String slid, String stuid) {
        return bChapterRepository.selectScoreListBySlIdAndStuId(slid, stuid);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public void deleteScoerInfo(String chapterId, String studentCode) {
//        String studentId = bStudentRepository.queryStudentIdByCode(studentCode);
//        int result = bChapterRepository.updateScoerInfo(chapterId,studentId);
//        if(result<1)
        throw new ServiceException(ResultServiceEnums.SCORE_INFO_NOT_EXISTS);
    }


    @Override
    public ScoreInfoDTO getChapInfoById(ScoreInfoDTO scoreInfoDTO) throws Exception {

        return bChapterRepository.queryChapInfoById(scoreInfoDTO.getChapterId(), scoreInfoDTO.getStudentId());
    }


    /**
     * 递归方法复制文件夹及子目录
     *
     * @param oldPath
     * @param newPath
     */
    private void copyFileToSl(String oldPath, String newPath) throws IOException {

        File oldFile = new File(oldPath);
        File[] fs = oldFile.listFiles();
        File newFile = new File(newPath);
        if (!newFile.exists()) {
            newFile.mkdirs();
        }
        for (File f : fs) {
            if (f.isFile()) {
                Path link = Paths.get(newPath + File.separator + f.getName());
                Path existing = Paths.get(f.getPath());
                Files.createLink(link, existing);
                //fileCopy(f.getPath(),newPath+File.separator+f.getName()); //调用文件拷贝的方法
            } else if (f.isDirectory()) {
                copyFileToSl(f.getPath(), newPath + File.separator + f.getName());
            }
        }
    }

    private void fileCopy(String oldPath, String newPath) {
        BufferedReader br = null;
        PrintStream ps = null;
        try {
            br = new BufferedReader(new InputStreamReader(new FileInputStream(oldPath)));
            ps = new PrintStream(new FileOutputStream(newPath));
            String s = null;
            while ((s = br.readLine()) != null) {
                ps.println(s);
                ps.flush();
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (br != null) br.close();
                if (ps != null) ps.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public int updateChapterState(BChapter bChapter) {
        if (bChapter.getState().equals(StateEnum.OPEN.getCode())) {
            bChapter.setState(StateEnum.CLOSE.getCode());
        } else
            bChapter.setState(StateEnum.OPEN.getCode());
        int result = bChapterRepository.updateStateById(bChapter);

        if (!bChapter.getPid().equals("-1")) {
            List<String> state = bChapterRepository.queryChapterStateByPid(bChapter.getPid());
            if (state.size() == 1 && state.get(0).equals(StateEnum.CLOSE.getCode())) {
                bChapterRepository.updateStateByPid(bChapter.getPid(), StateEnum.CLOSE.getCode());
            } else {
                bChapterRepository.updateStateByPid(bChapter.getPid(), StateEnum.OPEN.getCode());
            }
        }
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public int updateAllChapterState(String slId) {
        List<String> list = bChapterRepository.selectStateBySlId(slId);
        String state = StateEnum.OPEN.getCode();
        if (list.size() == 1 && list.get(0).equals(state)) {
            state = StateEnum.CLOSE.getCode();
        }
        int result = bChapterRepository.updateAllStateBySlId(slId, state);
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public int updateTopChapter(BChapter bChapter) {
        int result = 0;
        BChapter lessChapter = bChapterRepository.getlessChapter(bChapter);
        int sort = bChapter.getSort();
        int lessSort = lessChapter.getSort();
        int data = bChapterRepository.updateSortById(bChapter.getBusinessId(), lessSort);
        int data2 = bChapterRepository.updateSortById(lessChapter.getBusinessId(), sort);
        if (data == data2) {
            result = 1;
        }
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public int updateDownChapter(BChapter bChapter) {
        int result = 0;
        BChapter bigChapter = bChapterRepository.getBigChapter(bChapter);
        int sort = bChapter.getSort();
        int bigSort = bigChapter.getSort();
        int data = bChapterRepository.updateSortById(bChapter.getBusinessId(), bigSort);
        int data2 = bChapterRepository.updateSortById(bigChapter.getBusinessId(), sort);
        if (data == data2) {
            result = 1;
        }
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public void reSort(String pid, String lessonId) {
        List<String> chapterIds = bChapterRepository.getChapterIds(pid, lessonId);
        if (chapterIds != null && chapterIds.size() > 0) {
            List<Map<String, Object>> params = new ArrayList<>();
            for (int i = 0; i < chapterIds.size(); i++) {
                Map<String, Object> map = new HashedMap();
                map.put("businessId", chapterIds.get(i));
                map.put("sort", i + 1);
                params.add(map);
            }
            //更新表中sort
            bChapterRepository.updateSort(params);
        }
    }
}
