package org.rcisoft.business.bcourse.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.map.HashedMap;
import org.apache.ibatis.annotations.Param;
import org.rcisoft.business.bcourse.dao.BCourseRepository;
import org.rcisoft.business.bcourse.dto.QueryCourseResDTO;
import org.rcisoft.business.bcourse.entity.BCourse;
import org.rcisoft.business.bcourse.service.BCourseService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Api(tags = "03 课程分类")
@RestController
@RequestMapping("/BCourse")
public class BCourseController extends PaginationController<BCourse> {

    @Autowired
    private BCourseService bCourseServiceImpl;
    @Autowired
    private BCourseRepository bCourseRepository;

    @ApiOperation(value = "301 查询课程分类", notes = "查询课程分类", response = QueryCourseResDTO.class)
    @ApiImplicitParam(name = "cName", value = "(课程分类名)模糊查询", required = true, dataType = "varchar")
    @PostMapping(value = "/queryCourse")
    public Result queryBCourse(CurUser curUser, @Valid String cName, BindingResult br) {
        BCourse bCourse = new BCourse();
        bCourse.setCName(cName);
        bCourse.setCorpId(curUser.getCorpId());
        Map<String, Object> map = new HashedMap();
        map.put("courseList", bCourseServiceImpl.queryBCourse(bCourse));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                map);
    }

    @ApiOperation(value = "302 根据条件分页查询", notes = "根据条件分页查询标签", response = QueryCourseResDTO.class)
    @ApiImplicitParam(name = "cName", value = "(课程分类名)模糊查询", required = false, dataType = "varchar")
    @GetMapping(value = "/queryBCourseByPagination")
    public Result queryBCourseByPagination(CurUser curUser, @Valid String cName, BindingResult br) {
        BCourse bCourse = new BCourse();
        bCourse.setCName(cName);
        bCourse.setCorpId(curUser.getCorpId());
        bCourseServiceImpl.queryBCourseByPagination(getPaginationUtility(), bCourse);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value = "303 查询所有一级课程分类", notes = "查询所有一级课程分类", response = QueryCourseResDTO.class)
    @ApiImplicitParam(name = "num", value = "查询一级分类数量", required = true, dataType = "int")
    @PostMapping(value = "/queryFirstBCourse")
    public Result queryFirstBCourse(CurUser curUser,int num) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCourseServiceImpl.queryFirstLevel(curUser.getCorpId(),num));
    }

    @ApiOperation(value = "304 逻辑删除", notes = "根据ID删除一条记录")
    @ApiImplicitParam(name = "businessId", value = "课程分类id", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(CurUser curUser, @Valid String businessId, BindingResult br) {
        PersistModel data = bCourseServiceImpl.removeCourse(curUser.getCorpId(),businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value = "305 修改课程分类名", notes = "根据ID更新课程名")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "课程分类id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "cName", value = "课程分类名称", required = true, dataType = "varchar")})
    @PostMapping(value = "/update")
    public Result update(CurUser curUser, @Valid BCourse bCourse, BindingResult br) {
        PersistModel data = bCourseServiceImpl.updateCourse(bCourse);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCourse.getBusinessId());
    }

    @ApiOperation(value = "306 添加课程分类", notes = "添加课程分类，不填pid添加一级课程分类")
    @ApiImplicitParams({@ApiImplicitParam(name = "pid", value = "上一级", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "cName", value = "课程分类名", required = true, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result insert(CurUser curUser, @Valid BCourse bCourse, BindingResult br) {
        UserUtil.setCurrentPersistOperation(bCourse);
        PersistModel data = bCourseServiceImpl.insertCourse(bCourse);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bCourse.getPid());
    }

    @ApiOperation(value = "307 课程分类排序",notes = "拼接格式{\"businessid\":\"pid,sort,level\",\"businessid\":\"pid,sort,level\"}")
    @ApiImplicitParam(name = "json", value = "前台将所有课程的id,pid,sort,level拼成json", required = true, dataType = "varchar")
    @PostMapping(value = "/sort")
    public Result sort(CurUser curUser, String json) {
        return Result.builder(bCourseServiceImpl.sortCourse(json),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                json);
    }

}
