package org.rcisoft.business.bdiscuss.controller;


/*固定导入*/
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bdiscuss.dto.BDiscussDto;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.bdiscuss.entity.BDiscuss;
import org.rcisoft.business.bdiscuss.service.BDiscussService;

import java.util.List;

/**
* Created by  on 2019-9-25 9:03:09.
*/
@Api(tags = "08 课程评论")
@RestController
@RequestMapping("bdiscuss")
public class BDiscussController extends PaginationController<BDiscuss> {

    @Autowired
    private BDiscussService bDiscussServiceImpl;


    @ApiOperation(value="801  添加", notes="添加")
    @ApiImplicitParams({@ApiImplicitParam(name = "lessonId", value = "课程ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "content", value = "评论内容", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pid", value = "上一级（评论别人的评论需要）", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentId", value = "评论人id", required = true, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(CurUser curUser, @Valid BDiscuss bDiscuss, BindingResult bindingResult) {
        PersistModel data = bDiscussServiceImpl.save(curUser,bDiscuss);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        bDiscuss);
    }

    @ApiOperation(value="802 根据课程id查询评论集合", notes="根据课程id查询评论", response = BDiscussDto.class)
    @GetMapping("{lessonId:\\w+}")
    public Result getDiscussByLessonId(CurUser curUser, @PathVariable String lessonId) {

        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bDiscussServiceImpl.getDiscussByLessonId(lessonId,curUser.getCorpId()));
    }

    @ApiOperation(value="逻辑删除", notes="逻辑删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = false, dataType = "varchar")})
    @PostMapping("/delete/{id:\\w+}")
    public Result delete(@PathVariable String id) {
        BDiscuss bDiscuss = new BDiscuss();
        bDiscuss.setBusinessId(id);
//        bDiscuss.setToken(getToken());
        PersistModel data = bDiscussServiceImpl.remove(bDiscuss);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{id:\\w+}")
    public Result update(@Valid BDiscuss bDiscuss, BindingResult bindingResult) {
//        bDiscuss.setToken(getToken());
        PersistModel data = bDiscussServiceImpl.merge(bDiscuss);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        bDiscuss);
    }

    @ApiOperation(value="查看单 ", notes="查看单 ", response = BDiscuss.class)
    @GetMapping("/detail/{id:\\w+}")
    public Result detail(@PathVariable String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        bDiscussServiceImpl.findById(id));
    }


}
