package org.rcisoft.business.bexternallink.controller;


/*固定导入*/

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.ibatis.annotations.Param;
import org.rcisoft.business.bbanner.entity.BBanner;
import org.rcisoft.business.bexternallink.dto.AddOrUpdateDTO;
import org.rcisoft.business.bexternallink.dto.FindListDTO;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.util.UserUtil;

import javax.validation.Valid;


import org.rcisoft.business.bexternallink.entity.BExternalLink;
import org.rcisoft.business.bexternallink.service.BExternalLinkService;

import java.util.List;

/**
 * Created by  on 2019-11-13 13:28:38.
 */
@Api(tags = "14 外部链接")
@RestController
@RequestMapping("bexternallink")
public class BExternalLinkController extends PaginationController<BExternalLink> {

    @Autowired
    private BExternalLinkService bExternalLinkServiceImpl;


    @ApiOperation(value = "1401 添加", notes = "添加")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(CurUser curUser, @Valid AddOrUpdateDTO addOrUpdateDTO, BindingResult bindingResult) {
        PersistModel data = bExternalLinkServiceImpl.save(addOrUpdateDTO);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                addOrUpdateDTO);
    }

    @ApiOperation(value = "1402 逻辑删除", notes = "逻辑删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = false, dataType = "varchar")})
    @PostMapping("/delete")
    public Result delete(CurUser curUser, @Valid String businessId, BindingResult bindingResult) {
        PersistModel data = bExternalLinkServiceImpl.remove(businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value = "1403 查单一查询", notes = "查单一查询", response = BExternalLink.class)
    @GetMapping("/detail")
    public Result detail(CurUser curUser, @Valid String businessId, BindingResult bindingResult) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bExternalLinkServiceImpl.findById(businessId));
    }

    @ApiOperation(value = "1404 分页查询外链", notes = "分页查询外链", response = BExternalLink.class)
    @GetMapping(value = "/queryBExternalLinkByPagination")
    public Result listByPagination(CurUser curUser, @Valid FindListDTO findListDTO, BindingResult bindingResult) {
        bExternalLinkServiceImpl.findAllByPagination(getPaginationUtility(), findListDTO);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value = "1405 外链重新排序", notes = "外链重新排序")
    @PostMapping(value = "/updateExternalLinkOrder")
    public Result updateExternalLinkOrder(CurUser curUser, @Param("bannerOrderMap") String orderMap, BindingResult bindingResult){
        int data = bExternalLinkServiceImpl.updateExternalLinkOrder(orderMap);
        return Result.builder(new PersistModel(data),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                orderMap);
    }
}
