package org.rcisoft.business.bnotice.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bnotice.dto.ReadDTO;
import org.rcisoft.common.util.feignDto.MTUserGetsReqDTO;
import org.rcisoft.common.util.feignDto.MTUserInfoRspDTO;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bnotice.dao.BNoticeRepository;
import org.rcisoft.business.bnotice.entity.BNotice;
import org.rcisoft.business.bnotice.service.BNoticeService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.*;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-11-28 11:18:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BNoticeServiceImpl implements BNoticeService {

    @Autowired
    private BNoticeRepository bNoticeRepository;

    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;

    /**
    * 保存 bNotice
    * @param bNoticeList
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(List<BNotice> bNoticeList){
        //增加操作
//        List<String> reciIds = bNotice.getRecipientIds();
//        if (null !=reciIds && reciIds.size() > 0){
//            List<BNotice> bNoticeList = new ArrayList<>();
//            for (String reciId: reciIds){
//                BNotice bNotice1 = bNotice;
//                UserUtil.setCurrentPersistOperation(bNotice1);
//                bNotice1.setRecipientId(reciId);
//                bNotice1.setReadFlag("0");
//                bNoticeList.add(bNotice1);
//            }
//            int line = bNoticeRepository.insertBach(bNoticeList);
//            return new PersistModel(line);
//        }
//        int line = bNoticeRepository.insertSelective(bNotice);
        int line = bNoticeRepository.insertBach(bNoticeList);
        return new PersistModel(line);
    }


    /**
    * 逻辑删除
    * @param bNotice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BNotice bNotice){
        UserUtil.setCurrentMergeOperation(bNotice);
        bNotice.setDeleted();
        int line = bNoticeRepository.logicalDelete(bNotice);
        return new PersistModel(line);
    }

    /**
    * 修改 bNotice
    * @param bNotice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BNotice bNotice){
        UserUtil.setCurrentMergeOperation(bNotice);
        int line = bNoticeRepository.updateByPrimaryKeySelective(bNotice);
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bNotice
    * @param id
    * @return
    */
    public BNotice findById(String id){
        return bNoticeRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bNotice
    * @param bNotice
    * @return
    */
    public List<BNotice> findAllByPagination(PageUtil<BNotice> paginationUtility,
        BNotice bNotice){
        List<BNotice> noticeList = bNoticeRepository.queryBNotices(bNotice);
        if (noticeList == null || noticeList.size() < 1){
            return new ArrayList<>();
        }

        Set<String> ids = new HashSet<>();
        noticeList.forEach(bNotice1 -> {
            if (StringUtils.isNotEmpty(bNotice1.getSenderId())){
                ids.add(bNotice1.getSenderId());
            }
        });

        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(bNotice.getCorpId());
        mtUserGetsReqDTO.setIds(new ArrayList<>(ids));
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            mtUserInfoRspDTOList.forEach(mtUserInfoRspDTO -> {
                if (StringUtils.isNotEmpty(mtUserInfoRspDTO.getId())){
                    noticeList.forEach(bNotice1 -> {
                        if (mtUserInfoRspDTO.getId().equals(bNotice1.getSenderId())){
                            bNotice1.setHeadPic(mtUserInfoRspDTO.getAvatar());
                            bNotice1.setSenderName(mtUserInfoRspDTO.getName());
                        }
                    });
                }
            });
        }

        return noticeList;
    }

    @Override
    public PersistModel setRead(ReadDTO readDTO) {

        List<String> idList = Arrays.asList(readDTO.getIds().split(","));

        readDTO.setIdList(idList);

        BNotice bNotice = new BNotice();

        UserUtil.setCurrentMergeOperation(bNotice);

        bNotice.setReadFlag("1");

        bNoticeRepository.setRead(readDTO,bNotice);

        return new PersistModel(1);
    }

}
