package org.rcisoft.business.bpersonvalue.service.impl;

import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.bpersonvalue.dto.PersonValueClientDto;
import org.rcisoft.business.bpersonvalue.dto.PersonValueDto;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bpersonvalue.dao.BPersonValueRepository;
import org.rcisoft.business.bpersonvalue.entity.BPersonValue;
import org.rcisoft.business.bpersonvalue.service.BPersonValueService;


import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-11-13 13:28:38.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BPersonValueServiceImpl implements BPersonValueService {

    @Autowired
    private BPersonValueRepository bPersonValueRepository;

    @Autowired
    private BLessonRepository bLessonRepository;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Override
    public List<BPersonValue> queryByPagination(PageUtil pageUtil, CurUser curUser,PersonValueClientDto dto) {
        if(dto.getBusinessId() == null || "".equals(dto.getBusinessId())){
            dto.setBusinessId(curUser.getUserId());
        }
        return bPersonValueRepository.queryBPersonValues(dto);
    }

    @Override
    public String getUserValue(String corpId, String businessId) {
        return bPersonValueRepository.getUserValue(corpId,businessId);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public int add(PersonValueDto personValueDto,String corpId) {
        BPersonValue bPersonValue = new BPersonValue();
        BeanUtils.copyProperties(personValueDto, bPersonValue);
        UserUtil.setCurrentPersistOperation(bPersonValue);

        try{
            Long.parseLong(personValueDto.getValue());
            bPersonValue.setValue(personValueDto.getValue().trim());
        }catch (Exception e){
            throw new ServiceException(ResultServiceEnums.NUM_IS_ERROR);
        }

        if(personValueDto.getValue()==null || "".equals(personValueDto.getValue())){
            throw new ServiceException(ResultServiceEnums.VALUE_NOT_NULL);
        }
        SysUser sysUser = sysUserMapper.selectByPrimaryKey(personValueDto.getUserId());
        if (sysUser == null){
            throw new ServiceException(ResultServiceEnums.COMPANY_NOT_EXISTS);
        }
        if("0".equals(personValueDto.getType())){
            bPersonValue.setEvent(sysUser.getName()+"发放积分");
        }else if("1".equals(personValueDto.getType())){
            bPersonValue.setEvent(sysUser.getName()+"扣除积分");

            long nowValue = bPersonValueRepository.findUserValue(personValueDto.getPersonId());
            if(nowValue < 0){
                throw  new ServiceException(ResultServiceEnums.VALUE_HAVE_ERROR);
            }

            //扣减时先判断 当前积分是否大于扣减积分
            Long x = bPersonValueRepository.judgeUserValue(personValueDto.getPersonId(),personValueDto.getValue());
            if(x < 0){
                throw  new ServiceException(ResultServiceEnums.VALUE_IS_ERROR);
            }
        }
        int line = bPersonValueRepository.add(bPersonValue);
        //积分变化后 s_user表中value字段重新统计
        bPersonValueRepository.updateUserValue(personValueDto.getPersonId(),null);
        return line;
    }
}
