package org.rcisoft.core.result;

import com.google.common.base.MoreObjects;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.rcisoft.core.util.UserUtil;
import org.springframework.core.annotation.Order;

import java.io.Serializable;

/**
 * 返回信息
 *
 * @author David
 */
@ApiModel(value = "通知请求响应")
public class NotiRet<T> implements Serializable {
    @Order(1)
    @ApiModelProperty(value = "错误码")
    private String code;

    @Order(2)
    @ApiModelProperty(value = "错误描述")
    private String message;

    @Order(3)
    @ApiModelProperty(value = "请求时间")
    private Long ctime;

    @Order(4)
    @ApiModelProperty(value = "请求编号")
    private String requestId;

    @Order(5)
    @ApiModelProperty(value = "响应数据")
    private T data;

    public NotiRet() {
        this(null);
    }

    public NotiRet(String code) {
        this(code, null);
    }

    public NotiRet(String code, String message) {
        this.code = code;
        this.message = message;
        this.ctime = System.currentTimeMillis();
        if (UserUtil.getCurUser() != null) {
            this.requestId = String.valueOf(UserUtil.getCurUser().getRequireId());
        }
    }

    //构建成功返回
    public static NotiRet ok() {
        return new NotiRet(ResultCode.SUCCESS.getCode().toString(), "ok");
    }

    //构建系统异常返回
    public static NotiRet error() {
        return new NotiRet(ResultCode.ERROR.getCode().toString(), "false");
    }


    public String getCode() {
        return code;
    }

    public NotiRet<T> setCode(String code) {
        this.code = code;
        return this;
    }

    public String getMessage() {
        return message;
    }

    public NotiRet<T> setMessage(String message) {
        this.message = message;
        return this;
    }

    public Long getCtime() {
        return ctime;
    }

    public NotiRet<T> setCtime(Long ctime) {
        this.ctime = ctime;
        return this;
    }

    public String getRequestId() {
        return requestId;
    }

    public NotiRet<T> setRequestId(String requestId) {
        this.requestId = requestId;
        return this;
    }

    public T getData() {
        return data;
    }

    public NotiRet<T> setData(T data) {
        this.data = data;
        return this;
    }

    @Override
    public String toString() {
        return MoreObjects.toStringHelper(this)
                .add("\ncode", code)
                .add("\nmessage", message)
                .add("\nctime", ctime)
                .add("\nrequestId", requestId)
                .add("\ndata", data)
                .toString();
    }

}

